/*
	File name:	bmorph.c
	Type:		subroutines
	Author:		Gang Liu
	Date:		10/14/96
	Function:	Subroutines for some basic binary morphological
			operations.
*/
#include <stdio.h>	/* for NULL */
#include <memory.h>
#include <math.h>

#include "must.h"
#include "bmorph.h"

#ifndef	min
#define	min(x,y)	((x)<(y)?(x):(y))
#endif

#ifndef	max
#define	max(x,y)	((x)<(y)?(y):(x))
#endif

static void	draw_a_line(char *pixmap, int width,
			int x1, int y1, int x2, int y2, uchar val);

void	bopen(uchar **inimage, int width, int height, uchar fg, uchar bg,
		BSE *se, uchar **outimage)
{
	int	i;
	berosion(inimage, width, height, fg, bg, se, outimage);
	for(i=0;i<height;i++)
		memcpy(inimage[i], outimage[i], width*sizeof(char));
	bdilation(inimage, width, height, fg, bg, se, outimage);
	return;
}

void	bclose(uchar **inimage, int width, int height, uchar fg, uchar bg,
		BSE *se, uchar **outimage)
{
	int	i;
	bdilation(inimage, width, height, fg, bg, se, outimage);
	for(i=0;i<height;i++)
		memcpy(inimage[i], outimage[i], width*sizeof(char));
	berosion(inimage, width, height, fg, bg, se, outimage);
	return;
}

void	bdilation(uchar **inimage, int width, int height, uchar fg, uchar bg,
		BSE *se, uchar **outimage)
{
	int	i, j;
	int	x, y;
	int	xx, yy;

	for(i=0;i<height;i++)
		memcpy(outimage[i], inimage[i], width*sizeof(char));

	for(y=0;y<height;y++)
	    for(x=0;x<width;x++) {
		if( inimage[y][x] == fg )
			for(i=0;i<se->height;i++)
			    for(j=0;j<se->width;j++)
				if(se->SE[i][j]) {
					yy = y + i - se->y;
					xx = x + j - se->x;
					if(	yy>=0 && yy<height
					     && xx>=0 && xx<width )
						outimage[yy][xx] = fg;
				}
	    }

	return;
}

void	berosion(uchar **inimage, int width, int height, uchar fg, uchar bg,
		BSE *se, uchar **outimage)
{
	int	i, j;
	int	x, y;
	int	xx, yy;
	int	delete;

	for(i=0;i<height;i++)
		memcpy(outimage[i], inimage[i], width*sizeof(char));

	for(y=0;y<height;y++)
	    for(x=0;x<width;x++) {
		if( inimage[y][x] != fg ) continue;
		delete = 0;
		for(i=0;i<se->height && !delete;i++)
		    for(j=0;j<se->width;j++)
			if(se->SE[i][j]) {
				yy = y + i - se->y;
				xx = x + j - se->x;
				if(	yy<0 || yy>=height
				     || xx<0 || xx>=width ) {
					delete = 1;
					break;
				}
				if(inimage[yy][xx] != fg ) {
					delete = 1;
					break;
				}
			}
		if( delete ) outimage[y][x]=bg;
	    }

	return;
}

void	free_SE(BSE *se)
{
	if( se->SE != NULL ) free2d((void**)se->SE);
	se->SE = NULL;
	return;
}

int	block_SE(int width, int height, int cen_x, int cen_y, BSE *se)
{
	int	i, j;

	se->width = width;
	se->height = height;
	se->x = cen_x;
	se->y = cen_y;
	se->SE = (char**)alloc2d(se->width, se->height, sizeof(char));
	if(se->SE==NULL) return(-1);
	for(i=0;i<se->height;i++)
		for(j=0;j<se->width;j++)
			se->SE[i][j] = 1;
	return 0;
}

int	disk_SE(int width, int height, int cen_x, int cen_y, float R, BSE *se)
{
	int	i, j;
	int	itemp1, itemp2;
	float	ftemp;

	se->width = width;
	se->height = height;
	se->x = cen_x;
	se->y = cen_y;
	se->SE = (char**)alloc2d(se->width, se->height, sizeof(char));
	if(se->SE==NULL) return(-1);
	for(i=0;i<se->height;i++)
		for(j=0;j<se->width;j++) {
			itemp1 = i - se->y;
			itemp2 = j - se->x;
			ftemp = sqrt(itemp1*itemp1 + itemp2*itemp2);
			if( ftemp <= R )
				se->SE[i][j] = 1;
			else
				se->SE[i][j] = 0;
		}
	return 0;
}

int	line_SE(int width, int height, int cen_x, int cen_y,
		int x1, int y1, int x2, int y2, BSE *se)
{
	int	i, j;

	se->width = width;
	se->height = height;
	se->x = cen_x;
	se->y = cen_y;
	se->SE = (char**)alloc2d(se->width, se->height, sizeof(char));
	if(se->SE==NULL) return(-1);
	for(i=0;i<se->height;i++)
		for(j=0;j<se->width;j++) se->SE[i][j] = 0;
	draw_a_line(se->SE[0], se->width, x1, y1, x2, y2, 1);
	return 0;
}

static void	draw_a_line(char *pixmap, int width,
			int x1, int y1, int x2, int y2, uchar val)
{
	int	i, j;
	int	xx1, yy1, xx2, yy2;
	double	slope;

	/* draw the line in the pixmap */

	if( x1==x2) { /* vertical line */
		for(i=min(y1,y2);i<=max(y1,y2);i++)
			pixmap[ i*width + x1 ] = val;
		return;
	}

	if( y1==y2) { /* horizontal line */
		j = y1*width;
		for(i=min(x1,x2);i<=max(x1,x2);i++)
			pixmap[j+i] = val;
		return;
	}

	/* neither horizontal nor vertical */
	if( abs(y2-y1) >= abs(x2-x1) ) { /* more vertical than horizontal */
		if( y1 < y2 ) {
			yy1 = y1; yy2 = y2;
			xx1 = x1; xx2 = x2;
		}
		else {
			yy1 = y2; yy2 = y1;
			xx1 = x2; xx2 = x1;
		}

		slope = (double) (xx2-xx1) / (double) (yy2-yy1);
		for(i=yy1;i<=yy2;i++) {
			j = (i-yy1)*slope + xx1 + 0.5;
			pixmap[i*width+j] = val;
		}
	}
	else { /* more horizontal than vertical */
		if( x1 < x2 ) {
			xx1 = x1; xx2 = x2;
			yy1 = y1; yy2 = y2;
		}
		else {
			yy1 = y2; yy2 = y1;
			xx1 = x2; xx2 = x1;
		}

		slope = (double) (yy2-yy1) / (double) (xx2-xx1);
		for(i=xx1;i<=xx2;i++) {
			j = (i-xx1)*slope + yy1 + 0.5;
			pixmap[j*width+i] = val;
		}
	}

	return;
}

long	bcardinality(uchar **image, int width, int height, uchar fg, uchar bg)
{
	int	i, j;
	long	count=0;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			if(image[i][j]==fg) count++;

	return (count);
}

void	bcomplement(uchar **image, int width, int height, uchar fg, uchar bg)
{
	int	i, j;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			if(image[i][j]==fg)
				image[i][j]=bg;
			else
				if(image[i][j]==bg) image[i][j]=fg;
				/* do nothing is neither fg nor bg */
		}

	return;
}

void	bunion(uchar **image1, int width, int height, uchar fg, uchar bg,
		uchar **image2)
{
	int	i, j;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			if(image1[i][j]==fg) continue;
			if(image2[i][j]==fg) image1[i][j]=fg;
		}

	return;
}

void	bintersection(uchar **image1, int width, int height, uchar fg, uchar bg,
		uchar **image2)
{
	int	i, j;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			if(image1[i][j]!=fg) continue;
			if(image2[i][j]!=fg) image1[i][j]=bg;
		}

	return;
}

void	bsubtraction(uchar **image1, int width, int height, uchar fg, uchar bg,
		uchar **image2)
{
	int	i, j;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			if(image1[i][j]!=fg) continue;
			if(image2[i][j]==fg) image1[i][j]=bg;
		}

	return;
}

#undef	min
#undef	max
