/*
	File name:	boundary.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/30/97
	Function:	Assuming the input PGM file constains a label map of
			connected components, this utility creates an output
			image in PGM format which contains the boundary of
			the connected components by setting all non-boundary
			pixels to 0.
*/
#include <stdio.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"

void	usage(char *cmd);
void	get_boundary(uchar **areamap, int width, int height, uchar **boundmap);

int	main(int argc, char *argv[])
{
	int	width, height;
	uchar	**image, **bound;

	char	*fname;
	int	i, j;

	if(argc<3) {
		usage(argv[0]);
		return -1;
	}

	fname=argv[1];
	i=ReadPGM(fname, &width, &height, &image);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	bound=(uchar**)alloc2d(width, height, sizeof(uchar));
	if(bound==NULL) myerror(-3, stderr, "Not enough memory.\n");

	get_boundary(image, width, height, bound);

	fname=argv[2];
	i=WritePGM(fname, width, height, bound);
	if(i) myerror(-4, stderr, "Error writing output.\n");

	free2d((void**)image);
	free2d((void**)bound);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <PGM1> <PGM2>\n", cmd);
fprintf(stderr, "<PGM1> is the input PGM file name.\n");
fprintf(stderr, "<PGM2> is the output PGM file name.\n");
fprintf(stderr, "\n");
return;
}

void	get_boundary(uchar **areamap, int width, int height, uchar **boundmap)
{
	point	neighborhood[8]={	{-1,-1},{0,-1},{1,-1},
					{-1,0},        {1,0},
					{-1,1},{0,1},{1,1}};
	int	i, j, k;
	int	x, y;
	uchar	val, isboundary;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			boundmap[i][j]=0;
			val=areamap[i][j];
			if(!val) continue;
			isboundary=0;
			for(k=0;k<8;k++) {
				y=i+neighborhood[k].y;
				x=j+neighborhood[k].x;
				if(y<0 || y>=height || x<0 || x>=width)
					continue;
				if(areamap[y][x]!=val) {
					isboundary=1;
					break;
				}
			}
			if(isboundary) boundmap[i][j]=val;
		}
}

