/*
	File name:	commentpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		05/05/97
	Function:	Extract/Insert/Delete comment lines in the header
			of a pgm file.
			For Extract, the comment lines are sent to stdout.
			For Insert/Delete, a new pgm file is created.

			When inserting, the existing comment lines are first
			deleted. So, to insert with existing preserved, first
			get the existing lines into a file, then edit the file
			to add new lines, and use insert and redirect the input
			to that file.
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "must.h"
#include "pgm.h"

#define	LINELEN	256

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	char	op, upperop;
	char	*fname;

	int	width, height;
	uchar	**image;

	int	status;
	int	i, j;

	FILE	*fp;
	char	cbuf[LINELEN];

	if(argc<3) {
		usage(argv[0]);
		return -1;
	}

	op=argv[1][0];
	upperop=toupper(op);
	fname=argv[2];

	switch(upperop) {
		case 'E':
			fp=fopen(fname, "r");
			if(fp==NULL) {
				fprintf(stderr, "Error openning input file\n");
				return -2;
			}
			fgets(cbuf, LINELEN, fp); /* magic number "P5\n" */
			if(strcmp(cbuf,"P5\n")) {
				fprintf(stderr, "Not a PGM file.\n");
				fclose(fp);
				return -2;
			}
			for(;;) {
				fgets(cbuf, LINELEN, fp);
				if(cbuf[0]!='#') break;
				fprintf(stdout, "%s", cbuf);
			}
			fclose(fp);
			return 0;

		case 'D':
		case 'I':
			if(argc<4) {
				usage(argv[0]);
				return -1;
			}
			i=ReadPGM(fname, &width, &height, &image);
			if(i) {
				fprintf(stderr, "Error reading input image.\n");
				return -3;
			}
			fname=argv[3];
			break;
		default:
			fprintf(stderr, "Invalid operation choice : %c\n", op);
			usage(argv[0]);
			return -2;
	}

	status = 0;
	switch(upperop) {
		case 'D':
			i=WritePGM(fname, width, height, image);
			if(i) status = -4;
			break;

		case 'I':
			fp=fopen(fname, "w");
			if(fp==NULL) {
				status = -5;
				break;
			}

			fprintf(fp, "%s\n", "P5");
			for(;;) {
				fgets(cbuf, LINELEN, stdin);
				if(feof(stdin)) break;
				fprintf(fp, "# %s", cbuf);
				i=strlen(cbuf)-1;
				if(cbuf[i]!='\n') fprintf(fp,"\n");
			}
			fprintf(fp, "%d %d\n", width, height);
			fprintf(fp, "%d\n", 255);

			i=fwrite(image[0], width, height, fp);
			j=fclose(fp);
			if(i!=height || j) status = -6;
			break;
	}

	if(status) fprintf(stderr, "Error writing output.\n");

	free2d((void**)image);
	return status;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <e|d|i> <PGM1> [PGM2]\n", cmd);
fprintf(stderr, "<e|d|i> choice of operation: e-extract d-delete i-insert\n");
fprintf(stderr, "<PGM1> is input PGM file, required for all operations.\n");
fprintf(stderr, "[PGM2] is output PGM file, required for d and i.\n");
fprintf(stderr, "For insert, comments read in from stdin; EOF (^D) to end.\n");
fprintf(stderr, "\n");
return;
}
