/*
	File name:	linepgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		09/08/97
	Function:	Take an ordered list of 2D locations and draw straight
			line segments in a PGM file to connect them.
			If they form a closed contour, the inside will be
			filled by a specified value.
*/
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "must.h"
#include "pgm.h"
#include "mygraphics.h"
#include "connect.h"

void	usage(FILE *fp, char *cmd);

int	main(int argc, char *argv[])
{
	int	width, height;
	int	fg, bg, fill;
	int	num;
	int	*X, *Y, xo, yo;
	char	*outname;

	uchar	**image;
	int	**labels;

	int	i, j, k;
	int	num_CC, bg_label;

	if(argc<11) {
		usage(stderr, argv[0]);
		return -1;
	}

	i =sscanf(argv[1], "%d", &width);
	i+=sscanf(argv[2], "%d", &height);
	i+=sscanf(argv[3], "%d", &fg);
	i+=sscanf(argv[4], "%d", &bg);
	i+=sscanf(argv[5], "%d", &fill);
	i+=sscanf(argv[argc-3], "%d", &xo);
	i+=sscanf(argv[argc-2], "%d", &yo);
	outname=argv[argc-1];
	num=argc-9;
	if(i!=7 || (num%2)!=0) {
		usage(stderr, argv[0]);
		return -1;
	}

	fg%=256; bg%=256; fill%=256;
	if(fg==bg) myerror(-2, stderr, "fg and bg must differ and in 0~255!\n");

	num/=2;
	X=(int*)malloc(num*sizeof(int));
	Y=(int*)malloc(num*sizeof(int));
	image=(uchar**)alloc2d(width, height, sizeof(uchar));
	labels=(int**)alloc2d(width, height, sizeof(int));
	if(X==NULL || Y==NULL || image==NULL || labels==NULL)
		myerror(-2, stderr, "Not enough memory.\n");

	i=0;
	for(j=0;j<num;j++) {
		i+=sscanf(argv[6+2*j], "%d", &(X[j]));
		i+=sscanf(argv[7+2*j], "%d", &(Y[j]));
	}
	if(i!=2*num) myerror(-2, stderr, "Error reading vertices.\n");

	i=0;
	for(j=0;j<num;j++)
		if(X[j]<0 || X[j]>=width || Y[j]<0 || Y[j]>=height) i=1;
	if(xo<0 || xo>=width || yo<0 || yo>=height) i=1;
	if(i) myerror(-1, stderr, "Invalid specification for location(s).\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) image[i][j]=1;

	for(i=0;i<num-1;i++)
		draw_a_line(image[0],width,height,X[i],Y[i],X[i+1],Y[i+1],0);

	i=connected_component(image[0], width, height, 4, labels[0], &num_CC);
	if(i) myerror(i, stderr, "connected_component() failed.\n");

	/*
	if(num_CC!=2)
		printf("Warning: %d connected components found.\n", num_CC);
	*/

	bg_label=labels[yo][xo];

	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			image[i][j]=((labels[i][j]==bg_label) ? bg : fill);

	for(i=0;i<num-1;i++)
		draw_a_line(image[0],width,height,X[i],Y[i],X[i+1],Y[i+1],fg);

	i=WritePGM(outname, width, height, image);
	if(i) myerror(-4, stderr, "Error saving output file.\n");

	free2d((void**)image);
	free2d((void**)labels);
	free(X); free(Y);

	return 0;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp, "Function of %s:\n", cmd);
fprintf(fp,
"This utility takes an ordered list of 2D locations and draw straight line\n\
in a PGM file segments to connect them. If they form a closed contour, the\n\
inside will be filled by a specified value.\n");
fprintf(fp, "\n");
fprintf(fp,
"Usage:\n\
\t %s <width> <height> <fg> <bg> <fill> <x1 y1> [<x2 y2> ...] <xo yo> <PGM>\n",
	cmd);
fprintf(fp, "<width> number of columns of the image;\n");
fprintf(fp, "<height> number of rows of the image;\n");
fprintf(fp, "<fg> is the color value for drawing the edges;\n");
fprintf(fp, "<bg> is the color value for filling the OUTside;\n");
fprintf(fp, "<fill> is the color value for filling the INside;\n");
fprintf(fp, "<x1 y1> is the location of the 1st vertex (both positive);\n");
fprintf(fp, "[<x2 y2> ...] additional vertices;\n");
fprintf(fp, "<xo yo> is one point that is OUTSIDE of the (closed) contour;\n");
fprintf(fp, "<PGM> is the name string of the output PGM image file.\n");
fprintf(fp, "\n");
fprintf(fp, "All input parameters except <PGM> are integer numbers:\n");
fprintf(fp, "fg, bg, and fill takes value in 0~255; fg and bg must differ;\n");
fprintf(fp, "For the locations, 0<=x<=width-1, 0<=y<=height-1.\n");
}

