/*
	File name:	localmaximapgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		09/26/97
	Function:	Mark in a PGM file the locations of local maxima
			of the input PGM image.
*/
#include <stdio.h>

#include "must.h"
#include "pgm.h"

typedef	uchar	entry;

void	usage(FILE *fp, char *cmd);
void	local_maxima_2d(entry **data, int width, int height, int half,
		char mark, char **map);

int	main(int argc, char *argv[])
{
	int	width, height;
	int	half, fg, bg;
	char	*inname, *outname;

	uchar	**image, **map;

	int	i, j;

	if(argc<6) {
		usage(stderr, argv[0]);
		return -1;
	}

	inname=argv[1];
	i =sscanf(argv[2], "%d", &half);
	i+=sscanf(argv[3], "%d", &fg);
	i+=sscanf(argv[4], "%d", &bg);
	outname=argv[5];
	if(i!=3) {
		usage(stderr, argv[0]);
		return -1;
	}

	fg%=256; bg%=256;
	if(fg==bg) myerror(-2, stderr, "fg and bg must differ and in 0~255!\n");

	i=ReadPGM(inname, &width, &height, &image);
	if(i) myerror(-3, stderr, "Error reading input PGM image file.\n");

	map=(uchar**)alloc2d(width, height, sizeof(uchar));
	if(map==NULL) myerror(-4, stderr, "Error allocating memory.\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) map[i][j]=bg;

	local_maxima_2d(image, width, height, half, fg, (char**)map);

	i=WritePGM(outname, width, height, map);
	if(i) myerror(-5, stderr, "Error saving output file.\n");

	free2d((void**)image);
	free2d((void**)map);

	return 0;
}

void	local_maxima_2d(entry **data, int width, int height, int half,
		char mark, char **map)
{
	int	i, j, m, n;
	entry	center, this, maxval, minval;

	for(i=half;i<height-half;i++)
		for(j=half;j<width-half;j++) {
			maxval=minval=center=data[i][j];
			for(m=i-half;m<=i+half;m++)
				for(n=j-half;n<=j+half;n++) {
					this=data[m][n];
					if(this<minval) minval=this;
					if(this>maxval) maxval=this;
				}
			if(center==maxval && maxval>minval) map[i][j]=mark;
		}
	return;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp, "Function of %s:\n", cmd);
fprintf(fp,
"This utility takes an input PGM file and gives an output PGM file which\n\
contains the bitmap of the local maxima of the input image.\n");
fprintf(fp, "\n");
fprintf(fp, "Usage:\n\t %s <PGM1> <size> <fg> <bg> <PGM2>\n", cmd);
fprintf(fp, "<PGM1> (char*) is the name of the input image file;\n");
fprintf(fp, "<size> (int) tells the neighborhood (2*size+1)x(2*size+1);\n");
fprintf(fp, "<fg> (int) is the value for marking the NON-MAXIMA locations;\n");
fprintf(fp, "<bg> (int) is the value for marking the MAXIMA locations;\n");
fprintf(fp, "<PGM2> (char*) is the name of the output image file.\n");
fprintf(fp, "\n");
}

