#include <math.h>	/* for sqrt(), atan2() */

static int	myint(float f);

#ifndef	min
#define min(a,b)	(((a)<(b))?(a):(b))
#endif
#ifndef	max
#define max(a,b)	(((a)<(b))?(b):(a))
#endif
#ifndef	abs
#define abs(a)		(((a)<0)?-(a):(a))
#endif

void	draw_a_line(char *pixmap, int width, int height,
			int x1, int y1, int x2, int y2, char val)
{
	int	i, j;
	int	xx1, yy1, xx2, yy2;
	double	slope;

	/* draw the line in the pixmap */

	if( x1==x2) { /* vertical line */
		if(x1>=0 && x1<width) {
			for(i=min(y1,y2);i<=max(y1,y2);i++)
				if(i>=0 && i<height)
					pixmap[ i*width + x1 ] = val;
			return;
		}
	}

	if( y1==y2) { /* horizontal line */
		if(y1>=0 && y1<height) {
			j = y1*width;
			for(i=min(x1,x2);i<=max(x1,x2);i++)
				if(i>=0 && i<width)
					pixmap[j+i] = val;
			return;
		}
	}

	/* neither horizontal nor vertical */
	if( abs(y2-y1) >= abs(x2-x1) ) { /* more vertical than horizontal */
		if( y1 < y2 ) {
			yy1 = y1; yy2 = y2;
			xx1 = x1; xx2 = x2;
		}
		else {
			yy1 = y2; yy2 = y1;
			xx1 = x2; xx2 = x1;
		}

		slope = (double) (xx2-xx1) / (double) (yy2-yy1);

		/*
		yy1 = max(0, yy1);
		yy2 = min(height-1, yy2);
		*/

		for(i=yy1;i<=yy2;i++) {
			if(i<0 || i>=height) continue;
			j = myint((i-yy1)*slope + xx1);
			if(j>=0 && j<width) pixmap[i*width+j] = val;
		}
	}
	else { /* more horizontal than vertical */
		if( x1 < x2 ) {
			yy1 = y1; yy2 = y2;
			xx1 = x1; xx2 = x2;
		}
		else {
			yy1 = y2; yy2 = y1;
			xx1 = x2; xx2 = x1;
		}

		slope = (double) (yy2-yy1) / (double) (xx2-xx1);

		/*
		xx1 = max(0, xx1);
		xx2 = min(width -1, xx2);
		*/

		for(i=xx1;i<=xx2;i++) {
			if(i<0 || i>=width) continue;
			j = myint((i-xx1)*slope + yy1);
			if(j>=0 && j<height) pixmap[j*width+i] = val;
		}
	}

	return;
}

void	draw_a_ellipse(char *pixmap, int width, int height,
			float x, float y, float a, float b, char val)
{
	int	i;
	float	aa, bb;
	float	ratio;
	int	loc1, loc2;
	int	left, right, top, bottom;

	aa = a*a;
	bb = b*b;

	left	= myint(max(0, x-a));
	right	= myint(min(width -1, x+a));
	top	= myint(max(0, y-b));
	bottom	= myint(min(height-1, y+b));

	if(a>b) {	/* horizontally elongated */
		ratio = b/a;
		for(i=left;i<=right;i++) {
			loc1 = myint(y - ratio*sqrt(aa - (i-x)*(i-x)));
			loc2 = myint(y + ratio*sqrt(aa - (i-x)*(i-x)));
			if(loc1>=0 && loc1<height) pixmap[loc1*width+i] = val;
			if(loc2>=0 && loc2<height) pixmap[loc2*width+i] = val;
		}
	}
	else {		/* vertically elongated */
		ratio = a/b;
		for(i=top;i<=bottom;i++) {
			loc1 = myint(x - ratio*sqrt(bb - (i-y)*(i-y)));
			loc2 = myint(x + ratio*sqrt(bb - (i-y)*(i-y)));
			if(loc1>=0 && loc1<width) pixmap[i*width+loc1] = val;
			if(loc2>=0 && loc2<width) pixmap[i*width+loc2] = val;
		}
	}

	return;
}

void	draw_a_circle(char *pixmap, int width, int height,
			float x, float y, float r, char val)
{
	int	i;
	float	rr;
	int	loc1, loc2;
	int	left, right, top, bottom;

	rr = r*r;

	left	= myint(max(0, x-r));
	right	= myint(min(width -1, x+r));
	top	= myint(max(0, y-r));
	bottom	= myint(min(height-1, y+r));

	for(i=left;i<=right;i++) {
		loc1 = myint(y - sqrt(rr - (i-x)*(i-x)));
		loc2 = myint(y + sqrt(rr - (i-x)*(i-x)));
		if(loc1>=0 && loc1<height) pixmap[loc1*width+i] = val;
		if(loc2>=0 && loc2<height) pixmap[loc2*width+i] = val;
	}

	for(i=top;i<=bottom;i++) {
		loc1 = myint(x - sqrt(rr - (i-y)*(i-y)));
		loc2 = myint(x + sqrt(rr - (i-y)*(i-y)));
		if(loc1>=0 && loc1<width) pixmap[i*width+loc1] = val;
		if(loc2>=0 && loc2<width) pixmap[i*width+loc2] = val;
	}

	return;
}

static	int	between(float a1, float a2, float a);

void	draw_a_sector(char *pixmap, int width, int height,
			float x, float y, float a, float b,
			float a1, float a2, char val)
{
	int	i;
	float	aa, bb;
	float	ratio;
	float	angle;
	int	loc1, loc2;
	int	left, right, top, bottom;
	float	x1, y1, x2, y2;
	float	ta1, ta2;

	aa = a*a;
	bb = b*b;

	ta1 = tan(a1);
	ta2 = tan(a2);

	x1 = a*b*sqrt(1.0/(aa*ta1*ta1+bb));
	x2 = a*b*sqrt(1.0/(aa*ta2*ta2+bb));
	if(cos(a1)<0)	/* in the 2nd or the 3rd quadrant */
		x1 = -x1;
	if(cos(a2)<0)	/* in the 2nd or the 3rd quadrant */
		x2 = -x2;

	y1 = x1*ta1;
	y2 = x2*ta2;

	x1 += x;
	x2 += x;
	y1 += y;
	y2 += y;

	draw_a_line(pixmap, width, height, x, y, x1+.5, y1+.5, val);
	draw_a_line(pixmap, width, height, x, y, x2+.5, y2+.5, val);

	left	= (int) (max(0, x-a-0.99) );
	right	= min(width -1, (int)(x+a+0.99));
	top	= (int) (max(0, y-b-0.99) );
	bottom	= min(height-1, (int)(y+b+0.99));

	/* if(a>b) {*/	/* horizontally elongated */
		ratio = b/a;
		if(abs(ratio-1)<.01) ratio=1.0;
		for(i=left;i<=right;i++) {
			loc1 = myint(y - ratio*sqrt(aa - (i-x)*(i-x)));
			loc2 = myint(y + ratio*sqrt(aa - (i-x)*(i-x)));
			if(loc1>=0 && loc1<height) {
				angle=atan2(loc1-y, i-x);
				if(between(a1, a2, angle))
					pixmap[loc1*width+i] = val;
			}
			if(loc2>=0 && loc2<height) {
				angle=atan2(loc2-y, i-x);
				if(between(a1, a2, angle))
				pixmap[loc2*width+i] = val;
			}
		}
	/*
	}
	else {*/		/* vertically elongated */
		ratio = a/b;
		if(abs(ratio-1)<.01) ratio=1.0;
		for(i=top;i<=bottom;i++) {
			loc1 = myint(x - ratio*sqrt(bb - (i-y)*(i-y)));
			loc2 = myint(x + ratio*sqrt(bb - (i-y)*(i-y)));
			if(loc1>=0 && loc1<width) {
				angle=atan2(i-y, loc1-x);
				if(between(a1, a2, angle))
					pixmap[i*width+loc1] = val;
			}
			if(loc2>=0 && loc2<width) {
				angle=atan2(i-y, loc2-x);
				if(between(a1, a2, angle))
					pixmap[i*width+loc2] = val;
			}
		}
	/*}*/

	return;
}

void	draw_a_pie(char *pixmap, int width, int height,
			float x, float y, float a,
			float a1, float a2, char val)
{
	int	i;
	float	aa;
	float	angle;
	int	loc1, loc2;
	int	left, right, top, bottom;
	float	x1, y1, x2, y2;
	float	ca1, ca2;
	float	ta1, ta2;

	aa = a*a;

	ca1 = cos(a1);
	ca2 = cos(a2);

	ta1 = tan(a1);
	ta2 = tan(a2);

	x1 = aa*abs(ca1);
	x2 = aa*abs(ca2);
	if(ca1<0)	/* in the 2nd or the 3rd quadrant */
		x1 = -x1;
	if(ca2<0)	/* in the 2nd or the 3rd quadrant */
		x2 = -x2;

	y1 = x1*ta1;
	y2 = x2*ta2;

	x1 += x;
	x2 += x;
	y1 += y;
	y2 += y;

	draw_a_line(pixmap, width, height, x, y, x1+.5, y1+.5, val);
	draw_a_line(pixmap, width, height, x, y, x2+.5, y2+.5, val);

	left	= myint(max(0, x-a));
	right	= myint(min(width -1, x+a));
	top	= myint(max(0, y-a));
	bottom	= myint(min(height-1, y+a));

	for(i=left;i<=right;i++) {
		loc1 = myint(y - sqrt(aa - (i-x)*(i-x)));
		loc2 = myint(y + sqrt(aa - (i-x)*(i-x)));
		if(loc1>=0 && loc1<height) {
			angle=atan2(loc1-y, i-x);
			if(between(a1, a2, angle))
				pixmap[loc1*width+i] = val;
		}
		if(loc2>=0 && loc2<height) {
			angle=atan2(loc2-y, i-x);
			if(between(a1, a2, angle))
			pixmap[loc2*width+i] = val;
		}
	}

	for(i=top;i<=bottom;i++) {
		loc1 = myint(x - sqrt(aa - (i-y)*(i-y)));
		loc2 = myint(x + sqrt(aa - (i-y)*(i-y)));
		if(loc1>=0 && loc1<width) {
			angle=atan2(i-y, loc1-x);
			if(between(a1, a2, angle))
				pixmap[i*width+loc1] = val;
		}
		if(loc2>=0 && loc2<width) {
			angle=atan2(i-y, loc2-x);
			if(between(a1, a2, angle))
				pixmap[i*width+loc2] = val;
		}
	}

	return;
}

static	int	between(float a1, float a2, float a)
{
#define	PI	3.1415927

	int	result;

	while(a1>  PI)	a1 -= 2*PI;
	while(a1<=-PI)	a1 += 2*PI;
	while(a2>  PI)	a2 -= 2*PI;
	while(a2<=-PI)	a2 += 2*PI;
	while(a >  PI)	a  -= 2*PI;
	while(a <=-PI)	a  += 2*PI;

	result = 0;
	if(a2 >= a1) {	/* regular situation, not including Pi */
		if(a1 <= a && a <= a2)	result = 1;
	}
	else {	/* a2<a1, must be that Pi lies between a1 and a2 */
		if(a1 <= a || a <= a2)	result = 1;
	}

	return result;

#undef	PI
}

static int	myint(float f)
{
	int	ff=floor(f);
	return ((f-ff >= 0.5) ? ff+1 : ff);
}

