/*
	File name:	otsupgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/12/97
	Function:	Read in an image in a PGM file and sends to stdout
			the threshold value given by the Otsu algorithm.
*/
#include <stdio.h>

#include "must.h"
#include "pgm.h"
#include "otsu.h"

#define	GRAY_LEVELS	256

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	width, height;
	uchar	**image;

	int	i, j;
	long	hist[GRAY_LEVELS];
	int	T;

	char	*inname;

	if(argc<2) {
		usage(argv[0]);
		return -1;
	}

	inname=argv[1];

	i=ReadPGM(inname, &width, &height, &image);
	if(i) {
		fprintf(stderr, "%d returned when reading %s.\n", i, inname);
		return -3;
	}

	for(i=0;i<GRAY_LEVELS;i++) hist[i]=0;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			hist[ image[i][j] ] ++;

	i=otsu(hist, GRAY_LEVELS, &T);
	if(i)	fprintf(stderr, "otsu() returned status %d.\n", i);
	else	fprintf(stdout, "%d\n", T);

	free2d((void**)image);

	return 0;
}

void	usage(char *cmd)
{
	fprintf(stderr, "Usage:\n\t %s <PGM>\n", cmd);
	fprintf(stderr, "<PGM> is the image file in PGM format\n");
	fprintf(stderr, "\n");
}
