/*
	File name:	subpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		04/28/97
	Function:	Cut a sub-image from a PGM file and save the result
			also in a PGM file.
*/
#include <stdio.h>
#include <time.h>

#include "must.h"
#include "pgm.h"

#ifndef	mymin
#define	mymin(x,y)	((x)<=(y)?(x):(y))
#define	mymax(x,y)	((x)>=(y)?(x):(y))
#endif

#define	TUPLE	5

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	width1, height1;
	uchar	**image1;

	int	inputs;
	int	x1, y1, x2, y2;
	int	width2, height2;
	uchar	**image2;

	int	i, j, k;

	char	*inname, *outname;
	FILE	*fp;
	time_t	now;

	if(argc<7) {

		usage(argv[0]);
		return -1;
	}

	inname=argv[1];
	i=ReadPGM(inname, &width1, &height1, &image1);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	inputs = (argc-2) / TUPLE; /* less command name and input file name */

	for(k=0;k<inputs;k++) {
		/* location and name of the k-th sub-image */
		x1=atoi(argv[2+k*TUPLE]);
		y1=atoi(argv[3+k*TUPLE]);
		x2=atoi(argv[4+k*TUPLE]);
		y2=atoi(argv[5+k*TUPLE]);
		outname=argv[6+k*TUPLE];

		x1=mymax(0,x1);
		x1=mymin(width1-1,x1);
		y1=mymax(0,y1);
		y1=mymin(height1-1,y1);

		x2=mymax(0,x2);
		x2=mymin(width1-1,x2);
		y2=mymax(0,y2);
		y2=mymin(height1-1,y2);

		if(x2<x1 || y2<y1)
			myerror(-3, stderr, "Invalid corner specification.\n");

		width2=x2-x1+1;
		height2=y2-y1+1;

		image2=(uchar**)alloc2d(width2, height2, sizeof(uchar));
		if(image2==NULL)
			myerror(-4, stderr, "Error allocating memory.\n");

		for(i=y1;i<=y2;i++)
			for(j=x1;j<=x2;j++)
				image2[i-y1][j-x1] = image1[i][j];

		time(&now);

		fp=fopen(outname, "w");
		if(fp==NULL)
			myerror(-5, stderr, "Error openning output file.\n");

		fprintf(fp, "P5\n");
		/* begin comments */
		fprintf(fp, "# Name: %s\n", outname);
		fprintf(fp, "# Produced by subpgm on %s", ctime(&now));
		fprintf(fp, "# Cropped from %s\n", inname);
		fprintf(fp,
		"#   by taking the (%dx%d) region starting at (%d,%d)\n",
				width2, height2, x1, y1);
		/* end comments */
		fprintf(fp, "%d %d\n", width2, height2);
		fprintf(fp, "255\n");

		i=fwrite(image2[0], width2, height2, fp);
		j=fclose(fp);
		if(i!=height2 || j)
			myerror(-6, stderr, "Error writing output.\n");

		free2d((void**)image2);
	}

	free2d((void**)image1);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "\nFunction of %s:\n\n", cmd);
fprintf(stderr,
"Given an input image and the locations of the left-top and right-bottom\n\
corners of a series of sub-images, along with file names for each sub-image,\n\
this utility produces a series of output image files cut from the input image.\
\n\n");

fprintf(stderr, "Usage: %s <fi> <l1 t1 r1 b1 fo1> [<l2 t2 r2 b2 fo2> ...]\n",
	cmd);
fprintf(stderr, "\t fi - name of the input image file (PGM);\n");
fprintf(stderr,
"\t l1  - column position of the upper-left corner of the 1st sub-image;\n");
fprintf(stderr,
"\t t1  - row    position of the upper-left corner of the 1st sub-image;\n");
fprintf(stderr,
"\t r1  - column position of the lower-right corner of the 1st sub-image;\n");
fprintf(stderr,
"\t b1  - row    position of the lower-right corner of the 1st sub-image;\n");
fprintf(stderr, "\t fo1  - name of the first output sub-image file (PGM);\n");
fprintf(stderr, "\n");
fprintf(stderr, "\t <l2 t2 r2 b2 fo2> ... \t additional sub-images;\n");
fprintf(stderr, "\n");
return;
}
